@@ -1,27 +1,11 @@
-/*
- * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
- *
- * Author:     zccrs <zccrs@live.com>
- *
- * Maintainer: zccrs <zhangjide@deepin.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
 #include "dguiapplicationhelper.h"
 #include "private/dguiapplicationhelper_p.h"
 #include "dplatformhandle.h"
-#include <util/DFontManager>
+#include <DFontManager>
 
 #include <QHash>
 #include <QColor>
@@ -36,6 +20,33 @@
 #include <QLoggingCategory>
 #include <QDir>
 #include <QLockFile>
+#include <QDirIterator>
+#include <QDesktopServices>
+
+#ifdef Q_OS_UNIX
+#include <QDBusError>
+#include <QDBusReply>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QProcess>
+#endif
+#include <QDir>
+#include <QLockFile>
+#include <QDirIterator>
+#include <QDesktopServices>
+
+#ifdef Q_OS_UNIX
+#include <QDBusError>
+#include <QDBusReply>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QProcess>
+#include <DPathBuf>
+#endif
 
 #include <private/qguiapplication_p.h>
 #include <qpa/qplatformservices.h>
@@ -46,6 +57,39 @@
 #include <unistd.h>
 #endif
 
+#ifdef Q_OS_UNIX
+class EnvReplaceGuard
+{
+public:
+    EnvReplaceGuard(const int uid);
+    ~EnvReplaceGuard();
+
+    char *m_backupLogName;
+    char *m_backupHome;
+    bool initialized = false;
+};
+
+EnvReplaceGuard::EnvReplaceGuard(const int uid)
+{
+    if (struct passwd *pwd = getpwuid(static_cast<__uid_t>(uid))) {
+        m_backupLogName = getenv("LOGNAME");
+        m_backupHome = getenv("HOME");
+
+        setenv("LOGNAME", pwd->pw_name, 1);
+        setenv("HOME", pwd->pw_dir, 1);
+        initialized = true;
+    }
+}
+
+EnvReplaceGuard::~EnvReplaceGuard()
+{
+    if (initialized) {
+        setenv("LOGNAME", m_backupLogName, 1);
+        setenv("HOME", m_backupHome, 1);
+    }
+}
+#endif
+
 DGUI_BEGIN_NAMESPACE
 
 #ifdef QT_DEBUG
@@ -61,6 +105,9 @@ Q_GLOBAL_STATIC(DFontManager, _globalFM)
 
 #define WINDOW_THEME_KEY "_d_platform_theme"
 
+/*!
+ @private
+ */
 class _DGuiApplicationHelper
 {
 public:
@@ -91,6 +138,43 @@ public:
     static DGuiApplicationHelper::HelperCreator creator;
 };
 
+class LoadManualServiceWorker : public QThread
+{
+public:
+    explicit LoadManualServiceWorker(QObject *parent = nullptr);
+    ~LoadManualServiceWorker() override;
+    void checkManualServiceWakeUp();
+
+protected:
+    void run() override;
+};
+
+LoadManualServiceWorker::LoadManualServiceWorker(QObject *parent)
+    : QThread(parent)
+{
+    if (!parent)
+        connect(qApp, &QGuiApplication::aboutToQuit, this, std::bind(&LoadManualServiceWorker::exit, this, 0));
+}
+
+LoadManualServiceWorker::~LoadManualServiceWorker()
+{
+}
+
+void LoadManualServiceWorker::run()
+{
+    QDBusInterface("com.deepin.Manual.Search",
+                   "/com/deepin/Manual/Search",
+                   "com.deepin.Manual.Search");
+}
+
+void LoadManualServiceWorker::checkManualServiceWakeUp()
+{
+    if (this->isRunning())
+        return;
+
+    start();
+}
+
 DGuiApplicationHelper::HelperCreator _DGuiApplicationHelper::creator = _DGuiApplicationHelper::defaultCreator;
 Q_GLOBAL_STATIC(_DGuiApplicationHelper, _globalHelper)
 
@@ -105,13 +189,6 @@ DGuiApplicationHelperPrivate::DGuiApplicationHelperPrivate(DGuiApplicationHelper
 
 void DGuiApplicationHelperPrivate::init()
 {
-    D_Q(DGuiApplicationHelper);
-
-    systemTheme = new DPlatformTheme(0, q);
-    // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
-    // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
-    appTheme = systemTheme;
-
     if (qGuiApp) {
         initApplication(qGuiApp);
     } else {
@@ -125,6 +202,15 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
 {
     D_Q(DGuiApplicationHelper);
 
+    if (!systemTheme) {
+        // 需要在QGuiApplication创建后再创建DPlatformTheme，否则DPlatformTheme无效.
+        // qGuiApp->platformFunction()会报警告，并返回nullptr.
+        systemTheme = new DPlatformTheme(0, q);
+        // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
+        // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
+        appTheme = systemTheme;
+    }
+
     // 跟随application销毁
     qAddPostRoutine(staticCleanApplication);
 
@@ -159,11 +245,8 @@ void DGuiApplicationHelperPrivate::staticInitApplication()
     if (!_globalHelper.exists())
         return;
 
-    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load()) {
-        // systemTheme未创建时说明DGuiApplicationHelper还未初始化
-        if (helper->d_func()->systemTheme)
-            helper->d_func()->initApplication(qGuiApp);
-    }
+    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load())
+        helper->d_func()->initApplication(qGuiApp);
 }
 
 void DGuiApplicationHelperPrivate::staticCleanApplication()
@@ -251,45 +334,45 @@ bool DGuiApplicationHelperPrivate::isCustomPalette() const
  */
 
 /*!
-  \enum Dtk::Gui::DGuiApplicationHelper::ColorType
+  \enum DGuiApplicationHelper::ColorType
   DGuiApplicationHelper::ColorType 定义了主题类型.
   
-  \value UnknownType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::UnknownType
   未知主题(浅色主题或深色主题)
   
-  \value LightType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::LightType
   浅色主题
   
-  \value DarkType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::DarkType
   深色主题
  */
 
 /*!
-  \enum Dtk::Gui::DGuiApplicationHelper::Attribute
+  \enum DGuiApplicationHelper::Attribute
   DGuiApplicationHelper::Attribute 定义了功能属性
   
-  \value UseInactiveColorGroup
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::UseInactiveColorGroup
   如果开启，当窗口处于Inactive状态时就会使用QPalette::Inactive的颜色，否则窗口将没有任何颜色变化。
   
-  \value ColorCompositing
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::ColorCompositing
   是否采用半透明样式的调色板。
   
-  \value ReadOnlyLimit
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::ReadOnlyLimit
   区分只读枚举。
   
-  \value IsDeepinPlatformTheme
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDeepinPlatformTheme
   获取当前是否使用deepin的platformtheme插件，platformtheme插件可以为Qt程序提供特定的控件样式，默认使用chameleon主题。
   
-  \value IsDXcbPlatform
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDXcbPlatform
   获取当前使用的是不是dtk的xcb窗口插件，dxcb插件提供了窗口圆角和阴影功能。
   
-  \value IsXWindowPlatform
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsXWindowPlatform
   获取当前是否运行在X11环境中。
   
-  \value IsTableEnvironment
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsTableEnvironment
   获取当前是否运行在deepin平板环境中，检测XDG_CURRENT_DESKTOP环境变量是不是tablet结尾。
   
-  \value IsDeepinEnvironment
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDeepinEnvironment
   获取当前是否运行在deepin桌面环境中，检测XDG_CURRENT_DESKTOP环境变量是不是deepin。
  */
 
@@ -310,7 +393,7 @@ void DGuiApplicationHelper::initialize()
 /*!
   \brief 创建 DGuiApplicationHelper 对象.
 
-  \a creator 函数指针
+  \param creator 函数指针
   \note 一定要先调用此函数,再使用 DGuiApplicationHelper::instance()
  */
 void DGuiApplicationHelper::registerInstanceCreator(DGuiApplicationHelper::HelperCreator creator)
@@ -350,14 +433,14 @@ DGuiApplicationHelper::~DGuiApplicationHelper()
 
   \note 取值范围均为 -100 ~ 100 ,当三原色参数为-100时，颜色为黑色，参数为100时，颜色为白色.
   以透明度( alphaFloat )为例,当参数为负数时基础色的 alphaFloat 值减少，现象偏向透明, 参数为正数alphaFloat 值增加，现象偏不透明
-  \a base 基础色
-  \a hueFloat 色调
-  \a saturationFloat 饱和度
-  \a lightnessFloat 亮度
-  \a redFloat 红色
-  \a greenFloat 绿色
-  \a blueFloat 蓝色
-  \a alphaFloat Alpha通道(透明度)
+  \param base 基础色
+  \param hueFloat 色调
+  \param saturationFloat 饱和度
+  \param lightnessFloat 亮度
+  \param redFloat 红色
+  \param greenFloat 绿色
+  \param blueFloat 蓝色
+  \param alphaFloat Alpha通道(透明度)
   \return 经过调整的颜色
  */
 QColor DGuiApplicationHelper::adjustColor(const QColor &base,
@@ -388,11 +471,50 @@ QColor DGuiApplicationHelper::adjustColor(const QColor &base,
     return new_color;
 }
 
+/*!
+  \brief 调整图片整体像素颜色.
+
+  \note 取值范围均为 -100 ~ 100 ,当三原色参数为-100时，颜色为黑色，参数为100时，颜色为白色.
+  以透明度( alphaFloat )为例,当参数为负数时基础色的 alphaFloat 值减少，现象偏向透明, 参数为正数alphaFloat 值增加，现象偏不透明
+  \param base 基础色
+  \param hueFloat 色调
+  \param saturationFloat 饱和度
+  \param lightnessFloat 亮度
+  \param redFloat 红色
+  \param greenFloat 绿色
+  \param blueFloat 蓝色
+  \param alphaFloat Alpha通道(透明度)
+  \return 经过调整的图片
+ */
+QImage DGuiApplicationHelper::adjustColor(const QImage &base, qint8 hueFloat, qint8 saturationFloat, qint8 lightnessFloat, qint8 redFloat, qint8 greenFloat, qint8 blueFloat, qint8 alphaFloat)
+{
+    if (base.isNull())
+        return base;
+
+    if (!hueFloat && !saturationFloat && !lightnessFloat && !redFloat
+            && !greenFloat && !blueFloat && !alphaFloat)
+        return base;
+
+    QImage dest = base;
+    for (int y = 0; y < dest.height(); ++y) {
+        const QRgb *rgb = reinterpret_cast<const QRgb *>(base.scanLine(y));
+        for (int x = 0; x < dest.width(); ++x) {
+            QColor base = QColor::fromRgba(rgb[x]);
+            if (base.alpha() == 0)
+                continue;
+            QColor color = adjustColor(base, hueFloat, saturationFloat, lightnessFloat,
+                                       redFloat, greenFloat, blueFloat, alphaFloat);
+            dest.setPixel(x, y, color.rgba());
+        }
+    }
+    return dest;
+}
+
 /*!
   \brief 将两种颜色混合，合成新的颜色.
 
-  \a substrate 底层颜色
-  \a superstratum 上层颜色
+  \param substrate 底层颜色
+  \param superstratum 上层颜色
   \return 混合颜色
  */
 QColor DGuiApplicationHelper::blendColor(const QColor &substrate, const QColor &superstratum)
@@ -491,7 +613,7 @@ static QColor dark_dpalette[DPalette::NColorTypes] {
 /*!
   \brief 根据主题获取标准调色板.
 
-  \a type 主题枚举值
+  \param type 主题枚举值
   \return 调色板
  */
 DPalette DGuiApplicationHelper::standardPalette(DGuiApplicationHelper::ColorType type)
@@ -659,9 +781,9 @@ static void generatePaletteColor_helper(DPalette &base, M role, DGuiApplicationH
 /*!
   \brief 获取调色板颜色.
 
-  \a base 调色板
-  \a role 色码
-  \a type 主题枚举值
+  \param base 调色板
+  \param role 色码
+  \param type 主题枚举值
   \sa QPalette::ColorRole
  */
 void DGuiApplicationHelper::generatePaletteColor(DPalette &base, QPalette::ColorRole role, DGuiApplicationHelper::ColorType type)
@@ -695,9 +817,9 @@ void DGuiApplicationHelper::generatePaletteColor(DPalette &base, QPalette::Color
   0.7 后作为 QPalette::Disabled 类的颜色使用, 调整为 0.6 后作为 QPalette::Inactive
   类的颜色使用.
 
-  \a base 被加工的调色板
-  \a role 加工的项
-  \a type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
+  \param base 被加工的调色板
+  \param role 加工的项
+  \param type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
  */
 void DGuiApplicationHelper::generatePaletteColor(DPalette &base, DPalette::ColorType role, DGuiApplicationHelper::ColorType type)
 {
@@ -711,8 +833,8 @@ void DGuiApplicationHelper::generatePaletteColor(DPalette &base, DPalette::Color
   同 generatePaletteColor, 将直接调用 generatePaletteColor 加工
   所有类型的调色板颜色.
 
-  \a base 被加工的调色板
-  \a type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
+  \param base 被加工的调色板
+  \param type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
  */
 void DGuiApplicationHelper::generatePalette(DPalette &base, ColorType type)
 {
@@ -741,7 +863,7 @@ void DGuiApplicationHelper::generatePalette(DPalette &base, ColorType type)
   根据颜色类型将使用 standardPalette 获取基础调色板数据, 在此基础上
   从 DPlatformTheme::fetchPalette 获取最终的调色板.
 
-  \a theme 平台主题对象
+  \param theme 平台主题对象
   \return 调色板数据
  */
 DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
@@ -779,7 +901,7 @@ DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
   \brief 设置是否将调色板的颜色改为半透明模式.
 
   一般用在主窗口背景为透明、模糊的程序中
-  \a on 是否开启
+  \param on 是否开启
  */
 void DGuiApplicationHelper::setUseInactiveColorGroup(bool on)
 {
@@ -789,7 +911,7 @@ void DGuiApplicationHelper::setUseInactiveColorGroup(bool on)
 /*!
   \brief 设置是否开启混合颜色.
 
-  \a on 是否开启
+  \param on 是否开启
  */
 void DGuiApplicationHelper::setColorCompositingEnabled(bool on)
 {
@@ -859,7 +981,7 @@ DPlatformTheme *DGuiApplicationHelper::applicationTheme() const
 /*!
   \brief 返回窗口级别的主题, 优先级高于 windowTheme 和 systemTheme.
 
-  \a window 主题对象对应的窗口
+  \param window 主题对象对应的窗口
   \return 平台主题对象
   \sa applicationTheme()
   \warning 已废弃, 不再对外暴露为特定窗口设置主题的接口
@@ -947,7 +1069,7 @@ DPalette DGuiApplicationHelper::applicationPalette() const
   的 themeName 和所有与 \a palette 相关的属性都不再生效.
   \warning 使用此方式设置的调色板将不会自动跟随活动色的变化
   \warning 如果使用过QGuiApplication::setPalette, 此方式可能不会生效
-  \a palette 要设置的调色板
+  \param palette 要设置的调色板
  */
 void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
 {
@@ -982,14 +1104,10 @@ void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
   2. 通过 setThemeType 固定调色板的类型
   3. 通过 QGuiApplication::setPalette 固定调色板, 需要注意此方法不可逆.
   否则将基于窗口所对应的 DPlatformTheme 获取调色板( fetchPalette).
-  \a window
+  \param window
   \return 调色板
-  \sa windowTheme
-  \sa fetchPalette
-  \sa standardPalette
-  \sa generatePalette
-  \sa applicationPalette
-  \warning 使用时要同时关注 paletteChanged, 收到此信号后可能需要重新获取窗口的调色板
+  \sa windowTheme(), fetchPalette(), standardPalette(), generatePalette(), applicationPalette()
+  \warning 使用时要同时关注 applicationPaletteChanged() , 收到此信号后可能需要重新获取窗口的调色板
   \warning 已废弃, 不再对外暴露控制窗口级别调色板的接口
  */
 DPalette DGuiApplicationHelper::windowPalette(QWindow *window) const
@@ -1032,7 +1150,7 @@ const DFontManager *DGuiApplicationHelper::fontManager() const
 
   转换的策略为：先将颜色转换为rgb格式，再根据 Y = 0.299R + 0.587G + 0.114B 的公式
   计算出颜色的亮度，亮度大于 191 时认为其为浅色，否则认为其为深色。
-  \a color 需要转换为主题的类型的颜色
+  \param color 需要转换为主题的类型的颜色
   \return 颜色类型的枚举值
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor &color)
@@ -1058,7 +1176,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor
   使用 QPalette::background 获取颜色的明亮度，将其转换为主题类型的枚举值。
   返回调色板的颜色类型
 
-  \a palette 调色板
+  \param palette 调色板
   \return 颜色类型的枚举值
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QPalette &palette)
@@ -1106,8 +1224,8 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::paletteType() const
 /*!
   \brief 设置DGuiApplicationHelper实例.
 
-  \a key 实例关键字
-  \a singleScope 实例使用范围
+  \param key 实例关键字
+  \param singleScope 实例使用范围
   \return 设置是否成功
   \note 此处所用到DGuiApplicationHelperPrivate::waitTime默认值为3000ms，可通过
   \note DGuiApplicationHelper::setSingleInstanceInterval设置
@@ -1192,6 +1310,7 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
     }
 
     if (new_server) {
+        qCDebug(dgAppHelper) << "===> new server <===" << _d_singleServer->serverName() << getpid();
         QObject::connect(_d_singleServer, &QLocalServer::newConnection, qApp, [] {
             QLocalSocket *instance = _d_singleServer->nextPendingConnection();
             // 先发送数据告诉新的实例自己收到了它的请求
@@ -1229,7 +1348,7 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
   \brief 设置从QLocalServer获取消息的等待时间.
 
   用于在重新创建DGuiApplicationHelper单例时，检测DGuiApplicationHelper单例是否存在且有响应
-  \a interval 等待时间，如 \a interval 为 -1 则没有超时一直等待，默认和 QLocalSocket 一致 3000ms
+  \param interval 等待时间，如 \a interval 为 -1 则没有超时一直等待，默认和 QLocalSocket 一致 3000ms
   \note 需要在 DGuiApplicationHelper::setSingleInstance 之前调用否则无效。
  */
 void DGuiApplicationHelper::setSingleInstanceInterval(int interval)
@@ -1241,13 +1360,122 @@ void DGuiApplicationHelper::setSingleInstanceInterval(int interval)
 /*!
   \brief 设置从QLocalServer获取消息的等待时间.
 
-  \a interval 等待时间，typo 请使用 DGuiApplicationHelper::setSingleInstanceInterval
+  \param interval 等待时间，typo 请使用 DGuiApplicationHelper::setSingleInstanceInterval
  */
 void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
 {
     DGuiApplicationHelperPrivate::waitTime = interval;
 }
 
+/*!
+ * \brief Determine whether it's a user manual for this application.
+ * \return
+ */
+bool DGuiApplicationHelper::hasUserManual() const
+{
+#ifdef Q_OS_LINUX
+    auto loadManualFromLocalFile = [=]() -> bool {
+        const QString appName = qApp->applicationName();
+        bool dmanBinaryExists = false;
+        bool dmanDataExists = false;
+        const QString sysPath = qgetenv("PATH");
+        auto binPath = sysPath.split(":");
+        for (const auto path : binPath) {
+            if (QFile::exists(QStringList {path, "dman"}.join(QDir::separator()))) {
+                dmanBinaryExists = true;
+                break;
+            }
+        }
+
+        // search all subdirectories
+        const QString xdgDataPath = qgetenv("XDG_DATA_DIRS");
+        auto dataPath = xdgDataPath.split(":");
+        for (const auto path : dataPath) {
+            QString strManualPath = QStringList {path, "deepin-manual"}.join(QDir::separator());
+
+            QDirIterator it(strManualPath, QDirIterator::Subdirectories);
+            while (it.hasNext()) {
+                QFileInfo file(it.next());
+                if (file.isDir() && file.fileName().contains(appName, Qt::CaseInsensitive)) {
+                    dmanDataExists = true;
+                    break;
+                }
+
+                if (file.isDir())
+                    continue;
+            }
+        }
+
+        return  dmanBinaryExists && dmanDataExists;
+    };
+
+    QDBusConnection conn = QDBusConnection::sessionBus();
+    if (conn.interface()->isServiceRegistered("com.deepin.Manual.Search")) {
+        QDBusInterface manualSearch("com.deepin.Manual.Search",
+                                    "/com/deepin/Manual/Search",
+                                    "com.deepin.Manual.Search");
+        if (manualSearch.isValid()) {
+            QDBusReply<bool> reply = manualSearch.call("ManualExists", qApp->applicationName());
+            return reply.value();
+        } else {
+            return loadManualFromLocalFile();
+        }
+    } else {
+        static LoadManualServiceWorker *manualWorker = new LoadManualServiceWorker;
+        manualWorker->checkManualServiceWakeUp();
+
+        return loadManualFromLocalFile();
+    }
+#else
+    return false;
+#endif
+}
+
+bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<QString> &translateDirs, const QList<QLocale> &localeFallback)
+{
+    DCORE_USE_NAMESPACE;
+
+    QList<QString> dirs = translateDirs;
+    const QList<DPathBuf> defaultDirPrefix {
+        qApp->applicationDirPath(),
+        QDir::currentPath()
+    };
+    for (auto item : defaultDirPrefix)
+        dirs << item.join("translations").toString();
+
+    QStringList missingQmfiles;
+    for (const auto &locale : localeFallback) {
+        QStringList translateFilenames {QString("%1_%2").arg(fileName).arg(locale.name())};
+        const QStringList parseLocalNameList = locale.name().split("_", QString::SkipEmptyParts);
+        if (parseLocalNameList.length() > 0)
+            translateFilenames << QString("%1_%2").arg(fileName).arg(parseLocalNameList.at(0));
+
+        for (const auto &translateFilename : translateFilenames) {
+            for (const auto &dir : dirs) {
+                DPathBuf path(dir);
+                QString translatePath = (path / translateFilename).toString();
+                if (QFile::exists(translatePath + ".qm")) {
+                    qDebug() << "load translate" << translatePath;
+                    auto translator = new QTranslator(qApp);
+                    translator->load(translatePath);
+                    qApp->installTranslator(translator);
+                    qApp->setProperty("dapp_locale", locale.name());
+                    return true;
+                }
+            }
+
+            // fix english does not need to translation.
+            if (locale.language() != QLocale::English)
+                missingQmfiles << translateFilename + ".qm";
+        }
+    }
+
+    if (missingQmfiles.size() > 0) {
+        qWarning() << fileName << "can not find qm files" << missingQmfiles;
+    }
+    return false;
+}
+
 void DGuiApplicationHelper::setAttribute(DGuiApplicationHelper::Attribute attribute, bool enable)
 {
     if (attribute < Attribute::ReadOnlyLimit) {
@@ -1285,10 +1513,8 @@ bool DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::Attribute attri
 
 /*!
   \brief DGuiApplicationHelper::setThemeType.
-  \obsolete
-
-  同 setPaletteType， 已废弃，请不要再使用。
-  \a themeType 主题类型.
+  \deprecated 同 setPaletteType， 已废弃，请不要再使用。
+  \param themeType 主题类型.
  */
 void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeType)
 {
@@ -1304,7 +1530,7 @@ void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeT
   \note 主动设置调色板颜色类型的操作会导致程序不再使用 DPlatformTheme 中调色板相关的数据, 也
   包括窗口级别的 windowTheme 所对应的 DPlatformTheme, 届时设置 DPlatformTheme
   的 themeName 和所有与 palette 相关的属性都不再生效.
-  \a paletteType 主题类型的枚举值
+  \param paletteType 主题类型的枚举值
  */
 void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType paletteType)
 {
@@ -1325,6 +1551,52 @@ void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType pale
     Q_EMIT paletteTypeChanged(paletteType);
 }
 
+/*!
+ * \brief Open manual for this application.
+ */
+void DGuiApplicationHelper::handleHelpAction()
+{
+    if (!hasUserManual()) {
+        return;
+    }
+#ifdef Q_OS_LINUX
+    QString appid = qApp->applicationName();
+
+    // new interface use applicationName as id
+    QDBusInterface manual("com.deepin.Manual.Open",
+                          "/com/deepin/Manual/Open",
+                          "com.deepin.Manual.Open");
+    QDBusReply<void> reply = manual.call("ShowManual", appid);
+    if (reply.isValid())  {
+        return;
+    }
+    // fallback to old interface
+    QProcess::startDetached("dman", QStringList() << appid);
+#else
+    qWarning() << "not support dman now";
+#endif
+}
+
+void DGuiApplicationHelper::openUrl(const QString &url)
+{
+#ifdef Q_OS_UNIX
+    // workaround for pkexec apps
+    bool ok = false;
+    const int pkexecUid = qEnvironmentVariableIntValue("PKEXEC_UID", &ok);
+
+    if (ok)
+    {
+        EnvReplaceGuard _env_guard(pkexecUid);
+        Q_UNUSED(_env_guard);
+
+        QDesktopServices::openUrl(url);
+    }
+    else
+#endif
+    {
+        QDesktopServices::openUrl(url);
+    }
+}
 DGUI_END_NAMESPACE
 
 #include "moc_dguiapplicationhelper.cpp"
